/* eslint-disable no-console */
/* eslint-disable */
import {
  Http, Metadata, IDictionary,
  Loader,
} from '@zeedhi/core';
import {
  TextInput, Card, Select, Text,
  TreeGrid, Form, DialogService, AlertService,
  Button,
} from '@zeedhi/common';
import { getRouteParams } from '../router';

export class HierarquiasController {
  public modalHierarquia = Loader.getInstance('ModalHierarquiaController');

  /* --------- Tela: /hierarquias ---------- */

  // Busca todas as hierarquias e monta os cards
  public getHierarchies() {
    const cardObject = Metadata.getInstance<Card>('externalCard');
    cardObject.children = [];

    Http.get('/getHierarchies').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        cardObject.children.push((this.hierarchyCard(database.NRHIERARQUIAFUNC, database.NMHIERARQUIAFUNC, this.setStatusHierarchy(database.IDATIVO))));
      });
    });
  }

  // Seta a cor do status da vistoria: On ou Off
  public setStatusHierarchy(idativo: string) {
    return ((idativo === 'S') ? '#008000' : '#FF0000');
  }

  // Insere uma nova hierarquia
  public insertHierarchy() {
    const nmhierarquiafunc = Metadata.getInstance<TextInput>('hierarchyName').value;

    Http.post('/insertHierarchy', { nmhierarquiafunc }).then((response: IDictionary) => {
      if (response.data.dataset.info.error != "Inserido com sucesso") {
        DialogService.show({
          name: 'errorDialogLocal',
          type: 'error',
          title: 'Erro!',
          text: 'Não foi possível inserir uma nova hierarquia.',
        });
      }
      else {
        AlertService.show({
          name: 'confirmInsert',
          text: 'Hierarquia inserida com sucesso',
          color: 'success',
          top: true,
          left: false,
        });
      }
    });
    this.modalHierarquia.destroyNewHierarchyModal();
    this.modalHierarquia.closeNewHierarchyModal();
    setTimeout(() => {
      this.getHierarchies();
    }, 750);
  }

  // Buscar hierarquia
  public searchForHieararchy({ component }: {component : TextInput}) {
    const cardObject = Metadata.getInstance<Card>('externalCard');
    const searchObject = component.value.toLowerCase();
    cardObject.children.map((child) => {
      const titleCompare = child.name.substr(5);
      const lowerTitleCompare = titleCompare.toLowerCase();
      if (lowerTitleCompare.includes(searchObject)) {
        child.isVisible = true;
      } else {
        child.isVisible = false;
      }
    });
  }

  // Ativa a hierarquia
  public activateHierarchy(idHierarchy: number) {
    const cardObject = Metadata.getInstance<Card>('externalCard');

    Http.get(`/activateHierarchy?idHierarchy=${idHierarchy}`).then((response: IDictionary) => {
      if (response.data.dataset.info.error != "Ativado com sucesso") {
        DialogService.show({
          name: 'errorDialogLocal',
          type: 'error',
          title: 'Erro!',
          text: 'Não foi possível ativar hierarquia.',
        });
      }
      else {
        AlertService.show({
          name: 'confirmDelete',
          text: 'Hierarquia ativada.',
          color: 'success',
          top: true,
          left: false,
        });
      }
    });
    setTimeout(() => {
     this.getHierarchies();
    }, 750);
  }

  // Card de hierarquia da tela principal
  public hierarchyCard(nrhierarquiafunc: number, nmhierarquiafunc: string, statusColor: string) {
    return {
      name: `card_${nmhierarquiafunc}`,
      component: 'ZdCard',
      elevation: '2',
      cssClass: 'zd-my-2',
      isVisible: true,
      children: [
        {
          name: 'rowCard',
          component: 'ZdRow',
          justify: '',
          children: [
            {
              name: `status_${nrhierarquiafunc}`,
              component: 'ZdButton',
              iconName: 'mdi-circle',
              icon: true,
              color: statusColor,
              events: {
                click: () => this.activateHierarchy(nrhierarquiafunc),
              },
            },
            {
              name: `name_${nrhierarquiafunc}`,
              component: 'ZdText',
              text: nmhierarquiafunc,
              tag: 'h3',
              cssClass: 'zd-col-11',
            },
            {
              name: `enter_${nrhierarquiafunc}`,
              component: 'ZdButton',
              to: `/hierarquias/detalhes/${nrhierarquiafunc}`,
              icon: true,
              iconName: 'mdi-eye',
            },
          ],
        },
      ],
    };
  }

  /* --------- Tela: /hierarquias/detalhes ---------- */

  public emptyArray: IDictionary[] = [];

  // id da rota
  public getRouteId () {
    const routeParams = getRouteParams();
    const { id } = routeParams;
    return id;
  }

  // Array dos selects do modal Adicionar Usuário
  public selectsInfo: IDictionary = {
    userName: [],
    userParentName: [],
  };

  // Array onde serão salvos os usuários da hierarquia
  public gridInfo: IDictionary[] = [];

  public lowerHierarchyUsers: IDictionary = [];

  // Valores que serão armazenados no grid da hierarquias
  public treeGridUsers(nrUser: number, nmUser:string, nrParent: number, nrId: number) {
    return {
      value: nrUser,
      userParent: (nrParent === null) ? '' : nrParent,
      userName: nmUser,
      id: nrId,
    };
  }

  // Protótipo dos dados que serão salvos nos arrays do dicionários do modal
  public mountSelectData(idUser: number, nameUser: string) {
    return {
      name: nameUser,
      value: idUser,
    };
  }

  // Inicializa todas as informações do modal
  public getInfo() {
    this.getHierarchyUsers();
    this.getHierarchyUsersParents();
  }

  private deleteUserEvents: any = {
    click: () => {
      this.deleteHierarchyUser();
      DialogService.hide();
    },
  };

  private cancelDeleteEvents: any = {
    click: () => {
      DialogService.hide();
    },
  };

  // Salvando no data do component para exibir na tela
  public setSelectData({ component }: { component:Select }) {
    component.datasource.data = this.selectsInfo[component.name];
  }

  // Busca os detalhes da hierarquia selecionada
  public async getHierarchyDetails() {
    const gridObject = Metadata.getInstance<TreeGrid>('usersGrid');
    const id  = this.getRouteId();
    this.gridInfo = [];

    await Http.get(`/getHierarchyDetails?nrhierarquiafunc=${id}`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        this.gridInfo.push(this.treeGridUsers(database.NRFUNCIONARIO, database.NMFUNCIONARIO, database.NRFUNCIONARIOPAI, database.NRFUNCIONARIOSHIERARQ));
        Metadata.getInstance<Text>('title').text = database.NMHIERARQUIAFUNC;
      });
      gridObject.datasource.updateData(this.gridInfo);
    });
  }

  // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia, e seus possíveis pais
  public getHierarchyUsers() {
    const id  = this.getRouteId();
    Http.get(`/getHierarchyUsers?nrhierarquiafunc=${id}`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        this.selectsInfo.userName.push(this.mountSelectData(database.NRFUNCIONARIO, database.NMFUNCIONARIO));
      });
    });
  }

  // Busca uma lista personalizada de usuários que podem ser pais
  public getHierarchyUsersParents() {
    const id  = this.getRouteId();
    Http.get(`/getHierarchyUsersParents?nrhierarquiafunc=${id}`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        this.selectsInfo.userParentName.push(this.mountSelectData(database.NRFUNCIONARIO, database.NMFUNCIONARIO));
      });
    });
  }

  // Adiciona um novo usuário/funcionário na hierarquia
  public async insertHierarchyUser() {
    const gridObject = Metadata.getInstance<TreeGrid>('usersGrid');
    if (Metadata.getInstance<Form>('UserHierarchyform').validate()) {
      const id  = this.getRouteId();
      const params = {
        NRHIERARQUIAFUNC: Number(id),
        NRFUNCIONARIO: Metadata.getInstance<Select>('userName').value,
        NRFUNCIONARIOPAI: Metadata.getInstance<Select>('userParentName').value,
      };

      // Reload não funciona
      await Http.post('/insertHierarchyUser', { params }).then((response: IDictionary) => {
      if (response.data.dataset.info.error != "Inserido com sucesso") {
        DialogService.show({
          name: 'errorDialogLocal',
          type: 'error',
          title: 'Erro!',
          text: 'Não foi possível adicionar usuário à hierarquia.',
        });
      }
      else {
        AlertService.show({
          name: 'confirmInsert',
          text: 'Usuário adicionado.',
          color: 'success',
          top: true,
          left: false,
        });
      }
      });
      await gridObject.datasource.updateData(this.emptyArray);
      this.getHierarchyDetails();
      this.modalHierarquia.destroyNewUserHierarchyModal();
      this.modalHierarquia.closeNewUserHierarchyModal();
    }
  }

  // Abre o modal com um pai pré selecionado
  public openModalNewUser() {
    Promise.all([this.modalHierarquia.openNewUserHierarchyModal()]).then(() => {
      const gridObject = Metadata.getInstance<TreeGrid>('usersGrid');
      const userParentSelect = Metadata.getInstance<Select>('userParentName');
      gridObject.selectRow(gridObject.datasource.currentRow, false);
      if (gridObject.datasource.currentRow) {
        userParentSelect.setValue(gridObject.datasource.currentRow.value);
        userParentSelect.disabled = true;
      }
    });
  }

  // Habilita a visibilidade dos botôes quando clica na linha
  public isDetailsActionVisible({ row }: any) {
    const gridObject = Metadata.getInstance<TreeGrid>('usersGrid');
    return row.id === gridObject.datasource.currentRow.id;
  }

  // Deletar usuário e seus filhos da hierarquia
  public async deleteHierarchyUser() {
    const gridObject = Metadata.getInstance<TreeGrid>('usersGrid');
    this.selectUsersToDelete(gridObject.datasource.currentRow.value);
    const params = this.lowerHierarchyUsers;

    await Http.post('/deleteHierarchyUser', { params }).then((response: IDictionary) => {
      if (response.data.dataset.info.error != "Deletado com sucesso") {
        DialogService.show({
          name: 'errorDialogLocal',
          type: 'error',
          title: 'Erro!',
          text: 'Não foi possível deletar usuários.',
        });
      }
      else {
        AlertService.show({
          name: 'confirmDelete',
          text: 'Usuário(s) excluído(s) com sucesso.',
          color: 'success',
          top: true,
          left: false,
        });
      }
    });
    this.lowerHierarchyUsers = [];

    gridObject.datasource.updateData(this.emptyArray);
    this.getHierarchyDetails();
  }

  // Confirmar exclusão
  public confirmUserDelete() {
    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir Usuário',
      text: 'Este usuário poderá ser superior a outros usuários, portanto, ao excluir este usuário, todos os seus filhos também serão excluídos. Deseja confirmar a exclusão?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.cancelDeleteEvents,
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.deleteUserEvents,
        },
      ],
    });
  }

  // Monta o array de usuários que serão excluidos
  public selectUsersToDelete(nrfuncionario: number) {
    this.lowerHierarchyUsers.push(nrfuncionario);

    const gridObject = Metadata.getInstance<TreeGrid>('usersGrid');

    gridObject.datasource.data.forEach((user: IDictionary) => {
      if (nrfuncionario === user.userParent) this.selectUsersToDelete(user.value);
    });
  }
  
  // Busca o ID(nr) da hierarquia ativa e salva no local storage
  public getActiveHierarchyId() {
    Http.get('/getActiveHierarchyId').then((response: IDictionary) => {
      const nrhierarquiafunc = response.data.dataset.info[0].NRHIERARQUIAFUNC;
      window.localStorage.setItem('nrhierarquiafunc', nrhierarquiafunc);
    })
  }
  
  // Monta o array de usuários da ramificação abaixo
  public async mountHierarchyUsersArray() {
    const id = localStorage.getItem('nrhierarquiafunc');
    const nrfuncionario = localStorage.getItem('UserLogado'); // mandar esse userlogado na primeira chamada

    await Http.get(`/getHierarchyDetails?nrhierarquiafunc=${id}`).then((response: IDictionary) => {
      this.gridInfo = [];
      response.data.dataset.info.forEach((database: IDictionary) => {
       this.gridInfo.push(this.treeGridUsers(database.NRFUNCIONARIO, database.NMFUNCIONARIO, database.NRFUNCIONARIOPAI, database.NRFUNCIONARIOSHIERARQ));
      });
      this.lowerHierarchyUsers = [];
    });

    this.gatherLowerHierarchyUsers(Number(nrfuncionario));
    return this.lowerHierarchyUsers;
  }

  // Salva no array os usuários daquela ramificação
  public gatherLowerHierarchyUsers(nrfuncionario: number) {
    this.lowerHierarchyUsers.push(nrfuncionario);

    this.gridInfo.forEach((user: IDictionary) => {
      if (nrfuncionario === user.userParent) this.gatherLowerHierarchyUsers(user.value);
    });
  }
}
